
#
# Copyright (c) 2006-2014 Infiscale, Inc. All rights reserved
#


package Perceus::Datastore;
use strict;
use warnings;

use Perceus::Util;
use Perceus::Include;
#use Perceus::DB;
use Perceus::Network;
use File::Basename;
use DB_File;
use Fcntl;
use File::Path;
use Exporter;
use Text::ParseWords;
use Storable qw(freeze thaw);

our @ISA = ("Exporter");

our @EXPORT = qw(
    &gen_hostnames
);

=head1 NAME

Perceus::Datastore - Perceus datastore interface

=head1 ABOUT

Perl interface to the Perceus datastore. This interface will allow you to
create new database objects or open existing ones. This was first implemented in
2008 but for some reason it never made it past the svn. Will be making some major
updates for the 2014 devel season and integrating a much better backend.

=head1 SYNOPSIS

    use Perceus::Datastore;

    my $node = Perceus::Datastore->node($nodeid);
    $node->set("key1", "value1");
    $node->commit();

    my $value = $node->get("key1");


=cut

my %data;


if ( ! -d "$Perceus::Include::database" ) {
    mkpath("$Perceus::Include::database");
}

$data{"DB_OBJ"}{"NODE"} = tie(%{$data{"DB"}{"NODE"}}, "DB_File", "$Perceus::Include::database/Nodes.db", O_RDWR|O_CREAT, 0644, $DB_BTREE) or
    die("Could not attach to Berkeley BTree Database 'Nodes.db': $!\n");
$data{"DB_OBJ"}{"GROUP"} = tie(%{$data{"DB"}{"GROUP"}}, "DB_File", "$Perceus::Include::database/Groups.db", O_RDWR|O_CREAT, 0644, $DB_BTREE) or
    die("Could not attach to Berkeley BTree Database 'Groups.db': $!\n");
$data{"DB_OBJ"}{"VNFS"} = tie(%{$data{"DB"}{"VNFS"}}, "DB_File", "$Perceus::Include::database/Vnfs.db", O_RDWR|O_CREAT, 0644, $DB_BTREE) or
    die("Could not attach to Berkeley BTree Database 'Vnfs.db': $!\n");


=head1 FUNCTIONS

These functions are exported by this module to provide a fast way of
interfacing with all of the data.

=over 4
=cut


=item gen_hostnames

Generate a hash of hostnames that can be used to link to the ID

=cut
sub
gen_hostnames()
{
    my %return              = ();
    my @list                = ();

    foreach my $key ( keys %{$data{"DB"}{"NODE"}} ) {
        if ( $data{"DB"}{"NODE"}{$key} eq "" ) {
            next;
        }
        my %hash = %{ thaw($data{"DB"}{"NODE"}{"$key"}) };
        my $hostname = ${$hash{"HOSTNAME"}}[0] || $key;
        push(@{$return{$hostname}}, $key);
#        push(@{$return{$hostname}}, Perceus::Datastore->object("NODE", $key));
    }

    return(%return);
}



=item DB_exists(TYPE, ID)

Test to see if a DB entry exists.

=cut
sub
DB_exists($$)
{
    my $type                = shift;
    my $ID                  = shift;

    return(exists($data{"DB"}{"$type"}{"$ID"}));
}

=back

=head1 CONSTRUCTORS

=over 4
=cut

=item node(ID)

Returns a new Perceus::Datastore node object. This method must be passed with
a unique identifier string for the entry.

=cut
sub
node($$)
{
    my $proto                   = shift;
    my $class                   = ref($proto) || $proto;
    my $ID                      = shift;
    my $type                    = "NODE";
    my $self                    = ();

    %{$self}                    = ();

    $self->{"TYPE"}             = $type;
    $self->{"ID"}               = $ID;
    $self->{"REAL"}             = 1;

    bless($self, $class);

    if ( exists($data{"DB"}{"$type"}{"$ID"}) ) {
        $self->{"DATA"} = $data{"DB"}{"$type"}{"$ID"};
        &pprint(2, "Opening existing object for $type:$ID\n");
    } else {
        &pprint(2, "Creating new object for $type:$ID\n");
        $self->set("OBJECT", "$ID");
    }

    return($self);
}


=item group(ID)

Returns a new Perceus::Datastore group object. This method must be passed with
a unique identifier string for the entry.

=cut
sub
group($$)
{
    my $proto                   = shift;
    my $class                   = ref($proto) || $proto;
    my $ID                      = shift;
    my $type                    = "GROUP";
    my $self                    = ();

    %{$self}                    = ();

    $self->{"TYPE"}             = $type;
    $self->{"ID"}               = $ID;
    $self->{"REAL"}             = 1;

    bless($self, $class);

    if ( exists($data{"DB"}{"$type"}{"$ID"}) ) {
        $self->{"DATA"} = $data{"DB"}{"$type"}{"$ID"};
        &pprint(2, "Opening existing object for $type:$ID\n");
    } else {
        &pprint(2, "Creating new object for $type:$ID\n");
        $self->set("OBJECT", "$ID");
    }

    return($self);
}


=item vnfs(ID)

Returns a new Perceus::Datastore vnfs object. This method must be passed with
a unique identifier string for the entry.

=cut
sub
vnfs($$)
{
    my $proto                   = shift;
    my $class                   = ref($proto) || $proto;
    my $ID                      = shift;
    my $type                    = "VNFS";
    my $self                    = ();

    %{$self}                    = ();

    $self->{"TYPE"}             = $type;
    $self->{"ID"}               = $ID;
    $self->{"REAL"}             = 1;

    bless($self, $class);

    if ( exists($data{"DB"}{"$type"}{"$ID"}) ) {
        $self->{"DATA"} = $data{"DB"}{"$type"}{"$ID"};
        &pprint(2, "Opening existing object for $type:$ID\n");
    } else {
        &pprint(2, "Creating new object for $type:$ID\n");
        $self->set("OBJECT", "$ID");
    }

    return($self);
}

=item node_stack(ID)

Returns a new Perceus::Datastore node stack object. This method must be passed
with a unique identifier string for the entry. It will return a virtual
stack/configuration that has all of the Node, Group and VNFS configurations
combined into a final form.

=cut
sub
node_stack($$)
{
    my $proto                   = shift;
    my $class                   = ref($proto) || $proto;
    my $ID                      = shift;
    my $self                    = $class->node($ID);

    my %hash                    = $self->hash();
    my $type                    = "NODE";

    my @groups                  = qw(DEFAULT);
    my %seen                    = ();
    my $newself                 = ();

    %{$newself}                 = ();

    $newself->{"TYPE"}          = $type;
    $newself->{"ID"}            = $ID;
    $newself->{"VIRTUAL"}       = 1;

    if ( exists($hash{"VNFS"}) ) {
        #push(@groups, $hash{"VNFS"}[0]);
        my $vnfs = Perceus::Datastore->vnfs($hash{"VNFS"}[0]);
        if ( $vnfs ) {
            my %h = $vnfs->hash();
            %hash = &hash_merge(\%hash, \%h);
        }
    }

    if ( exists($hash{"GROUPS"}) ) {
        push(@groups, @{ $hash{"GROUPS"} });
    }

    foreach my $group ( @groups ) {
        if ( exists($seen{"$group"}) ) {
            next;
        } else {
            $seen{"$group"} = 1;
        }

        my $obj = Perceus::Datastore->group($group);

        if ( ! $obj ) {
            next;
        }

        my %grp = $obj->hash();

        if ( exists($grp{"GROUPS"}) ) {
            push(@groups, @{ $grp{"GROUPS"} });
        }

        %hash = &hash_merge(\%hash, \%grp);

    }

    $newself->{"DATA"} = freeze(\%hash);

    bless($newself, ref($self));

    return($newself);
}

=item tmp(ID)

Returns a new Perceus::Datastore volatile object which is just stored in memory
and thus is only available for as long as this processes is running. This
method must be passed with a unique identifier string for the entry.

note: This still requires a commit to write this object to the shared location
in memory if you want it to be accessible.

=cut
sub
tmp($$)
{
    my $proto                   = shift;
    my $class                   = ref($proto) || $proto;
    my $ID                      = shift;
    my $type                    = "TMP";
    my $self                    = ();

    %{$self}                    = ();

    $self->{"TYPE"}             = $type;
    $self->{"ID"}               = $ID;
    $self->{"TMP"}              = 1;

    bless($self, $class);

    if ( exists($data{"DB"}{"$type"}{"$ID"}) ) {
        $self->{"DATA"} = $data{"DB"}{"$type"}{"$ID"};
        &pprint(2, "Opening existing object for $type:$ID\n");
    } else {
        &pprint(2, "Creating new object for $type:$ID\n");
        $self->set("OBJECT", "$ID");
    }

    return($self);
}


=back

=head1 METHODS

=over 4
=cut

=item raw

Method to dump the object's raw frozen datastore stack.

=cut
sub
raw($)
{
    my $self                = shift;

    my $type                = $self->{"TYPE"};
    my $ID                  = $self->{"ID"};

    if ( ! exists($self->{"DATA"}) ) {
        return();
    }

    return($self->{"DATA"});
}

=item hash

Method to return a hash representation of the data object

=cut
sub
hash($)
{
    my $self                = shift;

    my $type                = $self->{"TYPE"};
    my $ID                  = $self->{"ID"};

    if ( ! exists($self->{"DATA"}) ) {
        return();
    }

    return(%{ thaw($self->{"DATA"}) });
}

# Desc:     Return the object's unique ID
sub
get_uuid($)
{
    my $self                = shift;

    return($self->{"UUID"});
}

=item get(key)

Method to print a scalar or list of values from object. If called in a
scalar context it will return the first item, otherwise it will return
an array of all values.

=cut
sub
get($$)
{
    my $self                = shift;
    my $key                 = uc(shift);

    my %hash = $self->hash();

    if ( ! exists($hash{"$key"}) ) {
        return();
    }

    if ( wantarray ) {
        return(@{$hash{"$key"}});
    } else {
        return(${$hash{"$key"}}[0]);
    }
}

# Desc:     Method to merge a hash config into the given object
# Params:   Hash reference to be merged
# Return:   true/false
sub
update($$)
{
    my $self                = shift;
    my $hash_update         = shift;
    my %hash_new            = ();

    my $type                = $self->{"TYPE"};
    my $ID                  = $self->{"ID"};

    %hash_new = $self->hash();

    foreach my $key ( keys %{$hash_update} ) {
        $key = uc($key);
        if ( $hash_update->{"$key"}  eq "" ) {
            delete($hash_new{"$key"});
        }
        foreach my $value ( &parse_line('\s+', 0, $hash_update->{"$key"}) ) {
            if ( $value eq "NULL" ) {
                delete($hash_new{"$key"});
            } else {
                if ( ! grep($_ eq $value, @{$hash_new{"$key"}} ) ) {
                    push(@{$hash_new{"$key"}}, $value);
                }
            }
        }
    }

    $self->{"DATA"} = freeze(\%hash_new);
    $self->{"MODIFIED"} = "1";

    return("1");
}


=item set(KEY, VALUE)

Method to set a key/value pair to a data object

=cut
sub
set($$$)
{
    my $self                = shift;
    my $key                 = uc(shift);
    my $value               = shift;
    my %hash_new            = ();

    my $type                = $self->{"TYPE"};
    my $ID                  = $self->{"ID"};

    %hash_new = $self->hash();

    if ( ! grep($_ eq $value, @{$hash_new{"$key"}} ) ) {
        push(@{$hash_new{"$key"}}, $value);
        &pprint(1, "Adding value: $type, $ID, $key, $value\n");
        $self->{"MODIFIED"} = "1";
    } else {
        &pprint(1, "Not adding value: $type, $ID, $key, $value\n");
    }

    $self->{"DATA"} = freeze(\%hash_new);


    return("1");
}

=item del(KEY, VALUE)

Method to delete information from an object. If you don't specify the
VALUE to delete, it will assume you want to delete all VALUES under
that key.

=cut
sub
del($$$)
{
    my $self                = shift;
    my $key                 = uc(shift);
    my $value               = shift;
    my %hash_new            = ();

    my $type                = $self->{"TYPE"};
    my $ID                  = $self->{"ID"};

    %hash_new = $self->hash();

    if ( defined($value) ) {
        @{$hash_new{"$key"}} = grep($_ ne $value, @{$hash_new{"$key"}} );
    } else {
        delete($hash_new{"$key"});
    }

    $self->{"DATA"} = freeze(\%hash_new);
    $self->{"MODIFIED"} = "1";

    return("1");
}


=item commit

Method to save the state of the object to disk

=cut
sub
commit($)
{
    my $self                = shift;
    my %tmp                 = ();
    my $obj                 = ();

    my $type                = $self->{"TYPE"};
    my $ID                  = $self->{"ID"};

    if ( exists($self->{"VIRTUAL"}) ) {
        return();
    }

    if ( exists($self->{"MODIFIED"}) ) {
        delete($self->{"MODIFIED"});
    } else {
        return();
    }

    &pprint(1, "commiting data to DB\n");

    # Push to in memory copy of data
    $data{"DB"}{"$type"}{"$ID"} = $self->{"DATA"};

    if ( exists($self->{"REAL"}) ) {
        $data{"DB_OBJ"}{"$type"}->sync();
    }

    return(1);
}


=item revert

Method to revert any changes since last commit in the object

=cut
sub
revert($)
{
    my $self                = shift;

    my $type                = $self->{"TYPE"};
    my $ID                  = $self->{"ID"};

    if ( ! exists($self->{"REAL"}) ) {
        return();
    }

    if ( exists($self->{"MODIFIED"}) ) {
        delete($self->{"MODIFIED"});
    }

    $self->{"DATA"}         = $data{"DB"}{"$type"}{"$ID"};

    return(1);
}


=back

=head1 SEE ALSO

Perceus::Network, Perceus::Link, Storable

=head1 COPYRIGHT

Perceus is copyright Arthur Stevens and Infiscale, Inc. and is GPL.

=cut



1;
