#!/usr/bin/perl

package Perceus::Packet;

use Time::HiRes ('gettimeofday');

use Perceus::Util;
use Exporter;
use strict;

my $HEADER_MAGIC = "PRCS20";
my $HEADER_FORMAT = "A6 A8 A8 A64 A64 A16 A4";
my $HEADER_LENGTH = length(pack($HEADER_FORMAT));

=head1 NAME

Perceus::Packet - Perceus packet interface

=head1 ABOUT

Perl interface to Perceus Packet formats. This module does not send packets,
rather assembles and parses them in a standardized fashion.

=head1 SYNOPSIS

    use Perceus::Packet;

    my $p = Perceus::Packet->new();

    $p->cid("23456789");
    $p->command("HELLO");
    $p->data("BLAH BLAH FDSAF DFAD");

    my $packet = $p->serialize();

    my $p1 = Perceus::Packet->new();
    $p1->parse(\$packet); 

    if ( $p1->satisified ) {
        my $command = $p1->command();
        my $data = $p1->data();

        my $send_packet = $p1->serialize();
    } else {
        print "Packet is missing data\n";
    }

    $p->clear();
    $p1->clear();

=head1 METHODS

=over 12
=cut


=item new()

Returns a new Perceus::Packet object.

=cut
sub
new($$)
{
    my $proto               = shift;
    my $class               = ref($proto) || $proto;
    my $self                = ();

    %{$self} = ();

    bless($self, $class);

    $self->{"CID"} = &rand_string(8);

    return $self;
}


=item unserialize()

Load a string into a packet object. Will return leftover data from string
that did not get added to the packet. So if multiple packets are in a single
string, this will trim off the top string the first packet. Call multiple
times to get all of the data until

=cut
sub
unserialize($$)
{
    my $self                    = shift;
    my $string                  = shift;
    my $return                  = ();
    my $magic                   = ();
    my $xid                     = ();
    my $cid                     = ();
    my $target                  = ();
    my $origin                  = ();
    my $command                 = ();
    my $length                  = ();
    my $data                    = ();

    if ( length($string) < $HEADER_LENGTH ) {
        &pprint(2, "String is smaller then allowed $HEADER_LENGTH\n");
        return($string);
    }

    (   $magic,
        $xid,
        $cid,
        $target,
        $origin,
        $command,
        $length ) = unpack($HEADER_FORMAT, $string);

    if ( $magic eq $HEADER_MAGIC ) {

        $data = substr($string, $HEADER_LENGTH, $length);

        &pprint(4, "magic:     ->$magic<-\n");
        &pprint(4, "xid:       ->$xid<-\n");
        &pprint(4, "cid:       ->$cid<-\n");
        &pprint(4, "target:    ->$target<-\n");
        &pprint(4, "origin:    ->$target<-\n");
        &pprint(4, "command:   ->$command<-\n");
        &pprint(4, "length:    ->$length<-\n");
        &pprint(4, "data:      ->$data<-\n");

        if ( $length == length($data) ) {
            $self->{"MAGIC"}    = $magic;
            $self->{"XID"}      = $xid;
            $self->{"CID"}      = $cid;
            $self->{"TARGET"}   = $target;
            $self->{"ORIGIN"}   = $origin;
            $self->{"COMMAND"}  = $command;
            $self->{"DATA"}     = $data;

            &pprint(4, "Message fill success\n");

            $return = substr($string, $HEADER_LENGTH + $length);

        }
    } else {
        &pprint(1, "Throwing out bad message: ->$string<-\n");
        if ( $string =~ /^.*?($HEADER_MAGIC.*)$/ ) {
            $return = $1;
        }
    }

    return($return);
}



=item fill()

Fills up a packet structure via a raw string. It will return with anything
leftover in the passed string that doesn't fill up the packet.

=cut
sub
fill($$)
{
    my $self                    = shift;
    my $string                  = shift;
    my $return                  = ();
    my $magic                   = ();
    my $xid                     = ();
    my $cid                     = ();
    my $target                  = ();
    my $origin                  = ();
    my $command                 = ();
    my $length                  = ();
    my $data                    = ();

    if ( length(${$string}) == 0 ) {
        &pprint(4, "Packet has zero length\n");
        return();
    }

    if ( length(${$string}) < $HEADER_LENGTH ) {
        &pprint(2, "Packet is smaller then $HEADER_LENGTH\n");
        return();
    }

    (   $magic,
        $xid,
        $cid,
        $target,
        $origin,
        $command,
        $length ) = unpack($HEADER_FORMAT, ${$string});

    if ( $magic eq $HEADER_MAGIC ) {

        $data = substr(${$string}, $HEADER_LENGTH, $length);

        &pprint(4, "magic:     ->$magic<-\n");
        &pprint(4, "xid:       ->$xid<-\n");
        &pprint(4, "cid:       ->$cid<-\n");
        &pprint(4, "target:    ->$target<-\n");
        &pprint(4, "origin:    ->$origin<-\n");
        &pprint(4, "command:   ->$command<-\n");
        &pprint(4, "length:    ->$length<-\n");
        &pprint(4, "data:      ->$data<-\n");

        if ( $length == length($data) ) {
            $self->{"MAGIC"}    = $magic;
            $self->{"XID"}      = $xid;
            $self->{"CID"}      = $cid;
            $self->{"TARGET"}   = $target;
            $self->{"ORIGIN"}   = $origin;
            $self->{"COMMAND"}  = $command;
            $self->{"DATA"}     = $data;

            &pprint(4, "Message fill success\n");

            # Pruning the length of data that we just filled the packet with
            ${$string} = substr(${$string}, $HEADER_LENGTH + $length);

            $return = 1;
        }
    } else {
        &pprint(3, "Throwing out bad message: ->". ${$string} ."<-\n");
        ${$string} = ();
    }

    return($return);
}


=item xid()

Will set or print the XID region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
xid($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 8 ) {
            $return = 1;
            $self->{"XID"} = $data;
        }

    } else {
        $return = $self->{"XID"};
    }

    return($return);
}

=item cid()

Will set or print the CID region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
cid($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 8 ) {
            $return = 1;
            $self->{"CID"} = $data;
        }

    } else {
        $return = $self->{"CID"};
    }

    return($return);
}


=item target()

Will set or print the TARGET region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
target($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 64 ) {
            $return = 1;
            $self->{"TARGET"} = $data;
        }

    } else {
        $return = $self->{"TARGET"};
    }

    return($return);
}


=item origin()

Will set or print the ORIGIN region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
origin($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 64 ) {
            $return = 1;
            $self->{"ORIGIN"} = $data;
        }

    } else {
        $return = $self->{"ORIGIN"};
    }

    return($return);
}

=item command()

Will set or print the COMMAND region of the packet. Optional argument will set.
Maximum size is 16 bytes.

=cut
sub
command($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 16 ) {
            $return = 1;
            $self->{"COMMAND"} = $data;
        }

    } else {
        $return = $self->{"COMMAND"};
    }

    return($return);
}

=item data()

Will set or print the DATA region of the packet. Optional argument will set.
Maximum size is 1024 bytes.

=cut
sub
data($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( defined($data) ) {

        if ( length($data) > 1024 ) {
            $data = substr(0, 1024);
            &pprint(1, "WARNING: trimmed data region of packet to 1024 bytes!\n");
        }

        $return = 1;
        $self->{"DATA"} = $data;
        $self->{"LENGTH"} = length($data);

    } else {
        $return = $self->{"DATA"};
    }

    return($return);
}


=item satisified()

If the packet is properly populated, this will return true.

=cut
sub
satisified($)
{
    my $self                = shift;

    return((exists($self->{"DATA"}) and exists($self->{"LENGTH"}) and $self->{"LENGTH"} == length($self->{"DATA"}) ? 1 : undef) );
}

=item clear()

Clear the object contents.

=cut
sub
clear($)
{
    my $self                = shift;

    %{$self} = ();
}

=item serialize()

Pack the object into a string suitable for sending over a socket

=cut
sub
serialize($)
{
    my $self                = shift;

    my $string = (pack($HEADER_FORMAT,
        $HEADER_MAGIC,
        &rand_string(8),
        $self->{"CID"} || "",
        $self->{"TARGET"} || "",
        $self->{"ORIGIN"} || "",
        $self->{"COMMAND"} || "",
        $self->{"LENGTH"} || length($self->{"DATA"}),
    ));

    $string .= $self->{"DATA"};

    return($string);
}

=back

=head1 SEE ALSO

Perceus::Network, Perceus::Link

=head1 COPYRIGHT

Perceus is copyright Arthur Stevens and Infiscale, Inc.

=cut

1;
