#!/usr/bin/perl

package Perceus::Server;

use Exporter;
use IO::Socket;

use Perceus::Util;
use Perceus::Socket;
use Perceus::Packet;

our @ISA = ("Exporter");

our @EXPORT = qw(
);

=head1 NAME

Perceus::Server - Socket interface to the Perceus server

=head1 ABOUT

Perl meta-interface to Perceus server process. It is a meta-interface because
it doesn't define any new functions or features. It simply calls several
other objects and methods together into a faster to access interface.

So instead of calling Perceus::Server, you can use Perceus::Packet and
Perceus::Socket directly and bypass this class completly.

=head1 SYNOPSIS

    use Perceus::Server;

    my $object = Perceus::Server->connect($PERCEUSD_HOSTNAME);
    my $retval;
    my @out;
    if ( ! $object->sendrecv("CONNECT", "$PROVISIND_HOSTNAME") ) {
        print "Sending command\n";
        ($retval, @out) = $object->sendrecv("RUN", "ls -l /");
    }
    foreach ( @out ) {
        chomp;
        print "->$_<-\n";
    }
    print "retval: $retval\n";


=head1 METHODS

=over 12
=cut


=item connect(HOSTNAME)

This is the main constructor for the Perceus::Server class. It will create a
new object and connect to a server (first argument if present or localhost if
no server address is specified).

=cut
sub
connect($$)
{
    my $proto               = shift;
    my $server              = shift;
    my $class               = ref($proto) || $proto;
    my $self                = ();

    %{$self} = ();

    bless($self, $class);

    $self->{"PACKET"} = Perceus::Packet->new();
    $self->{"SOCK"} = Perceus::Socket->connect($server);

    if ( ! defined($self->{"SOCK"}) or ! defined($self->{"PACKET"})) {
        return();
    }

    return $self;
}


=item sendrecv(COMMAND, DATA)

The send/receive method will send a properly formatted packet to the Perceus
server and then wait for the response. This method will block until the
server sends a RETVAL message.

The return value will either be a true/fail (0 or >0 respectively) if called
in a scalar context, otherwise it will respond with an array following the
true/fail that contains all of the STDOUT messages that the server sent.

=cut
sub
sendrecv($$)
{
    my $self                = shift;
    my $command             = shift;
    my $data                = shift;
    my $until               = shift || "RETVAL";
    my $retval              = 0;
    my @out                 = ();

    $self->{"PACKET"}->command($command);
    $self->{"PACKET"}->data($data);

    $self->{"SOCK"}->send($self->{"PACKET"}->serialize());

    do {
        $self->{"PACKET"}->clear();
        $string = $self->{"SOCK"}->recieve();
        while ( $self->{"PACKET"}->fill(\$string)) {
            if ( $self->{"PACKET"}->command() eq "STDOUT" ) {
                push(@out, $self->{"PACKET"}->data());
            }
        }
    } while ( $self->{"PACKET"}->command() ne $until );

    if ( wantarray ) {
        return($self->{"PACKET"}->data(), @out);
    } else {
        return($self->{"PACKET"}->data());
    }
}

=item send(COMMAND, DATA)

Similar to the send/receive method above but this method is non blocking
(which means, there is no way to know if the message was a success or even
if it was received properly.

=cut
sub
send($$$)
{
    my $self                = shift;
    my $command             = shift;
    my $data                = shift;

    $self->{"PACKET"}->command($command);
    $self->{"PACKET"}->data($data);

    $self->{"SOCK"}->send($self->{"PACKET"}->serialize());

}

=back

=head1 SEE ALSO

Perceus::Network, Perceus::Link

=head1 COPYRIGHT

Perceus is copyright Arthur Stevens and Infiscale, Inc.

=cut

1;
