#!/usr/bin/perl

package Perceus::Socket;

use Exporter;
use IO::Socket;
use Time::HiRes ('gettimeofday');

use Perceus::Util;
use Perceus::Include;
use Perceus::Packet;

our @ISA = ("Exporter");

our @EXPORT = qw(
);

=head1 NAME

Perceus::Socket - Perceus socket interface

=head1 ABOUT

Perl interface to Perceus network sockets.

=head1 SYNOPSIS

    use Perceus::Socket;

    my $object = Perceus::Socket->new();
    $object->send($packet);
    my $string = $object->receive();

=head1 METHODS

=over 12
=cut


=item new()

This will connect to a Perceus server socket. It takes an argument as the
server IP address to connect to. If the IP address provided is 127.0.0.1 or
if there is no argument specified, then it will attach to localhost.

=cut

sub
new($$)
{
    my $proto               = shift;
    my $server              = shift || "127.0.0.1";
    my $class               = ref($proto) || $proto;
    my $self                = ();
    my $sock                = ();

    %{$self} = ();

    bless($self, $class);

#    if ( $server eq "127.0.0.1" ) {
#        $sock = IO::Socket::UNIX->new(Peer => $Perceus::Include::perceusd_socket);
#
#    } elsif ( defined($server) ) {
        $sock = IO::Socket::INET->new(PeerAddr => $server,
                                      PeerPort => '9870',
                                      Proto    => 'tcp');
#    }

    $self->{"SOCK"} = $sock;

    my $string = $self->receive();

    $self->{"PACKET"} = Perceus::Packet->new();
    $self->{"PACKET"}->fill(\$string);

    if ( $self->{"PACKET"}->command() eq "HELLO" ) {
        &pprint(1, "DEBUG: Connected to Perceus server\n");
        $self->{"PACKET"}->data("Hello from Perceus client $Perceus::Include::ID");
        $self->send($self->{"PACKET"}->serialize());
    } else {
        die "Could not connect to Perceus server: ". $self->{"PACKET"}->command() ."\n";
    }

    return $self;
}

=item return_sock()

Return the socket object

=cut
sub
return_sock($)
{
    my $self                = shift;

    return($self->{"SOCK"});
}


=item connected()

Test to see if the socket is open

=cut
sub
connected($)
{
    my $self                = shift;

    return($self->{"SOCK"}->opened());
}


=item receive()

Recieve any available data from the socket

=cut
sub
receive($)
{
    my $self                = shift;
    my $num_read            = 0;
    my $buff_size           = 128;
    my $string              = ();
    my $sock                = $self->{"SOCK"};

    do {
        $ret = $self->{"SOCK"}->sysread($buff, $buff_size);
        $string .= $buff;
        if (defined($ret)) {
            $num_read += $ret;
        } else {
            $ret = 0;
        }
    } while ($ret == $buff_size);

    &pprint(1, "Recieved: $string\n");

    return($string);
}


=item send()

Send a string to the socket

=cut
sub
send($$)
{
    my $self                = shift;
    my $string              = shift;

    &pprint(1, "Sending:  $string\n");

    return(syswrite($self->{"SOCK"}, $string));
}


=item send_command()

Send a command and data message, and wait for the server to respond
with the retval.

=cut
sub
send_command($)
{
    my $self                = shift;
    my $command             = shift;
    my $data                = shift;
    my $done                = ();
    my $retval              = ();
    my @output              = ();
    my $packet              = $self->{"PACKET"};

    if ( defined($command) and defined($data) ) {

        $packet->command($command);
        $packet->data($data);
        $self->send($packet->serialize());

        while ( ! $done ) {

            my $packet_buff = $self->receive();

            while ( $packet_buff ) {
                $packet_buff = $packet->unserialize($packet_buff);
                if ( $packet->command() eq "RETVAL" ) {
                    $retval = $packet->data();
                    $done = 1;
                    last;
                } elsif ( $packet->command() eq "STDOUT" ) {
                    push(@output, $packet->data());
                }
            }
        }
    }

    return(wantarray ? ($retval, @output) : $retval);
}


=back

=head1 SEE ALSO

Perceus::Packet

=head1 COPYRIGHT

Perceus is copyright Arthur Stevens and Infiscale, Inc.

=cut

1;
